/**
 * A helper that implements TypeScript’s `Pick` utility type.
 *
 * @type {<T extends Partial<Record<PropertyKey, any>>, K extends (keyof T)[]>(obj: T, keys: K) => Pick<T, K[number]>}
 * @template {Partial<Record<PropertyKey, any>>} T
 * @template {keyof T} K
 */
export const pick = (/** @type {T} */ obj, /** @type {K[]} */ keys) => {
  const result = {}

  for (const key in obj) {
    // @ts-ignore
    if (keys.includes(key)) {
      // @ts-ignore
      result[key] = obj[key]
    }
  }

  // @ts-ignore
  return result
}

/**
 * A helper that implements TypeScript’s `Omit` utility type.
 *
 * @type {<T extends Partial<Record<PropertyKey, any>>, K extends (keyof T)[]>(obj: T, keys: K) => Omit<T, K[number]>}
 * @template {Partial<Record<PropertyKey, any>>} T
 * @template {keyof T} K
 */
export const omit = (/** @type {T} */ obj, /** @type {K[]} */ keys) => {
  const result = {}

  for (const key in obj) {
    // @ts-ignore
    if (!keys.includes(key)) {
      // @ts-ignore
      result[key] = obj[key]
    }
  }

  // @ts-ignore
  return result
}

export const tests = {
  pick: (/** @type {import('bun:test').Expect} */ expect) => {
    const obj = { a: 42, b: '42', c: 17 }

    expect(pick(obj, ['a', 'b'])).toEqual({ a: 42, b: '42' })
  },

  omit: (/** @type {import('bun:test').Expect} */ expect) => {
    const obj = { a: 42, b: '42', c: 17 }

    expect(omit(obj, ['c'])).toEqual({ a: 42, b: '42' })
  }
}
